import { Module, Provider } from '@nestjs/common';
import { CreateAdGrpcController } from './interface/grpc-controllers/create-ad.grpc.controller';
import { CqrsModule } from '@nestjs/cqrs';
import {
  AD_MESSAGE_PUBLISHER,
  AD_REPOSITORY,
  DATETIME_TRANSFORMER,
  PARAMS_PROVIDER,
  TIMEZONE_FINDER,
  TIME_CONVERTER,
} from './ad.di-tokens';
import { AdRepository } from './infrastructure/ad.repository';
import { DefaultParamsProvider } from './infrastructure/default-params-provider';
import { AdMapper } from './ad.mapper';
import { CreateAdService } from './core/application/commands/create-ad/create-ad.service';
import { TimezoneFinder } from './infrastructure/timezone-finder';
import { TimeConverter } from './infrastructure/time-converter';
import { FindAdByIdGrpcController } from './interface/grpc-controllers/find-ad-by-id.grpc.controller';
import { FindAdByIdQueryHandler } from './core/application/queries/find-ad-by-id/find-ad-by-id.query-handler';
import { PublishMessageWhenAdIsCreatedDomainEventHandler } from './core/application/event-handlers/publish-message-when-ad-is-created.domain-event-handler';
import { PrismaService } from './infrastructure/prisma.service';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { DateTimeTransformer } from './infrastructure/datetime-transformer';

const grpcControllers = [CreateAdGrpcController, FindAdByIdGrpcController];

const eventHandlers: Provider[] = [
  PublishMessageWhenAdIsCreatedDomainEventHandler,
];

const commandHandlers: Provider[] = [CreateAdService];

const queryHandlers: Provider[] = [FindAdByIdQueryHandler];

const mappers: Provider[] = [AdMapper];

const repositories: Provider[] = [
  {
    provide: AD_REPOSITORY,
    useClass: AdRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: AD_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];
const orms: Provider[] = [PrismaService];

const adapters: Provider[] = [
  {
    provide: PARAMS_PROVIDER,
    useClass: DefaultParamsProvider,
  },
  {
    provide: TIMEZONE_FINDER,
    useClass: TimezoneFinder,
  },
  {
    provide: TIME_CONVERTER,
    useClass: TimeConverter,
  },
  {
    provide: DATETIME_TRANSFORMER,
    useClass: DateTimeTransformer,
  },
];

@Module({
  imports: [CqrsModule],
  controllers: [...grpcControllers],
  providers: [
    ...eventHandlers,
    ...commandHandlers,
    ...queryHandlers,
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
    ...adapters,
  ],
  exports: [
    PrismaService,
    AdMapper,
    AD_REPOSITORY,
    PARAMS_PROVIDER,
    TIMEZONE_FINDER,
  ],
})
export class AdModule {}
