import { AD_MESSAGE_PUBLISHER, AD_REPOSITORY } from '@modules/ad/ad.di-tokens';
import { AdMapper } from '@modules/ad/ad.mapper';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import {
  CreateAdProps,
  DefaultAdProps,
  Frequency,
} from '@modules/ad/core/domain/ad.types';
import { AdRepository } from '@modules/ad/infrastructure/ad.repository';
import { PrismaService } from '@modules/ad/infrastructure/prisma.service';
import { ConfigModule } from '@nestjs/config';
import { EventEmitterModule } from '@nestjs/event-emitter';
import { Test } from '@nestjs/testing';

describe('Ad Repository', () => {
  let prismaService: PrismaService;
  let adRepository: AdRepository;

  const executeInsertCommand = async (table: string, object: any) => {
    const command = `INSERT INTO ${table} ("${Object.keys(object).join(
      '","',
    )}") VALUES (${Object.values(object).join(',')})`;

    await prismaService.$executeRawUnsafe(command);
  };
  const getSeed = (index: number, uuid: string): string => {
    return `'${uuid.slice(0, -2)}${index.toString(16).padStart(2, '0')}'`;
  };

  const baseUuid = {
    uuid: 'be459a29-7a41-4c0b-b371-abe90bfb6f00',
  };
  const baseOriginWaypointUuid = {
    uuid: 'bad5e786-3b15-4e51-a8fc-926fa9327ff1',
  };
  const baseDestinationWaypointUuid = {
    uuid: '4d200eb6-7389-487f-a1ca-dbc0e40381c9',
  };
  const baseUserUuid = {
    userUuid: "'113e0000-0000-4000-a000-000000000000'",
  };
  const driverAd = {
    driver: 'true',
    passenger: 'false',
    seatsProposed: 3,
    seatsRequested: 0,
    strict: 'false',
  };
  const punctualAd = {
    frequency: `'PUNCTUAL'`,
    fromDate: `'2023-01-01'`,
    toDate: `'2023-01-01'`,
    monTime: 'NULL',
    tueTime: 'NULL',
    wedTime: 'NULL',
    thuTime: 'NULL',
    friTime: 'NULL',
    satTime: 'NULL',
    sunTime: `'2023-01-01T07:00:00Z'`,
    monMargin: 900,
    tueMargin: 900,
    wedMargin: 900,
    thuMargin: 900,
    friMargin: 900,
    satMargin: 900,
    sunMargin: 900,
  };
  const originWaypoint = {
    position: 0,
    lon: 43.7102,
    lat: 7.262,
    locality: "'Nice'",
    postalCode: "'06000'",
    country: "'France'",
  };
  const destinationWaypoint = {
    position: 1,
    lon: 43.2965,
    lat: 5.3698,
    locality: "'Marseille'",
    postalCode: "'13000'",
    country: "'France'",
  };

  const createPunctualDriverAds = async (nbToCreate = 10) => {
    const adToCreate = {
      ...baseUuid,
      ...baseUserUuid,
      ...driverAd,
      ...punctualAd,
    };
    for (let i = 0; i < nbToCreate; i++) {
      adToCreate.uuid = getSeed(i, baseUuid.uuid);
      await executeInsertCommand('ad', adToCreate);
      await executeInsertCommand('waypoint', {
        uuid: getSeed(i, baseOriginWaypointUuid.uuid),
        adUuid: adToCreate.uuid,
        ...originWaypoint,
      });
      await executeInsertCommand('waypoint', {
        uuid: getSeed(i, baseDestinationWaypointUuid.uuid),
        adUuid: adToCreate.uuid,
        ...destinationWaypoint,
      });
    }
  };

  const mockMessagePublisher = {
    publish: jest.fn().mockImplementation(),
  };

  const mockLogger = {
    log: jest.fn(),
    warn: jest.fn(),
    error: jest.fn(),
  };

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      imports: [
        EventEmitterModule.forRoot(),
        ConfigModule.forRoot({ isGlobal: true }),
      ],
      providers: [
        PrismaService,
        AdMapper,
        {
          provide: AD_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
      ],
    })
      // disable logging
      .setLogger(mockLogger)
      .compile();

    prismaService = module.get<PrismaService>(PrismaService);
    adRepository = module.get<AdRepository>(AD_REPOSITORY);
  });

  afterAll(async () => {
    await prismaService.$disconnect();
  });

  beforeEach(async () => {
    await prismaService.ad.deleteMany();
  });

  describe('findOneById', () => {
    it('should return an ad', async () => {
      await createPunctualDriverAds(1);
      const result = await adRepository.findOneById(baseUuid.uuid, {
        waypoints: true,
      });

      expect(result.id).toBe(baseUuid.uuid);
    });
  });

  describe('create', () => {
    it('should create an ad', async () => {
      const beforeCount = await prismaService.ad.count();

      const createAdProps: CreateAdProps = {
        userId: 'b4b56444-f8d3-4110-917c-e37bba77f383',
        driver: true,
        passenger: false,
        frequency: Frequency.PUNCTUAL,
        fromDate: '2023-02-01',
        toDate: '2023-02-01',
        schedule: [
          {
            time: '12:05',
          },
        ],
        seatsProposed: 3,
        seatsRequested: 1,
        strict: false,
        waypoints: [
          {
            position: 0,
            address: {
              locality: 'Nice',
              postalCode: '06000',
              country: 'France',
              coordinates: {
                lon: 43.7102,
                lat: 7.262,
              },
            },
          },
          {
            position: 1,
            address: {
              locality: 'Marseille',
              postalCode: '13000',
              country: 'France',
              coordinates: {
                lon: 43.2965,
                lat: 5.3698,
              },
            },
          },
        ],
      };

      const defaultAdProps: DefaultAdProps = {
        driver: false,
        passenger: true,
        marginDuration: 900,
        seatsProposed: 3,
        seatsRequested: 1,
        strict: false,
      };

      const adToCreate: AdEntity = AdEntity.create(
        createAdProps,
        defaultAdProps,
      );
      await adRepository.insert(adToCreate);

      const afterCount = await prismaService.ad.count();

      expect(afterCount - beforeCount).toBe(1);
    });

    // it('should throw a UniqueConstraintException if ad already exists', async () => {
    //   await prismaService.ad.create({
    //     data: {
    //       uuid: uuid,
    //       password: bcrypt.hashSync(`password`, 10),
    //     },
    //   });

    //   const authenticationToCreate: AuthenticationEntity =
    //     await AuthenticationEntity.create(createAuthenticationProps);
    //   await expect(
    //     authenticationRepository.insert(authenticationToCreate),
    //   ).rejects.toBeInstanceOf(UniqueConstraintException);
    // });
  });
});
