import { TimeConverter } from '@modules/ad/infrastructure/time-converter';

describe('Time Converter', () => {
  it('should be defined', () => {
    const timeConverter: TimeConverter = new TimeConverter();
    expect(timeConverter).toBeDefined();
  });

  describe('localStringTimeToUtcStringTime', () => {
    it('should convert a paris time to utc time', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisTime = '08:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        parisTime,
        'Europe/Paris',
      );
      expect(utcDatetime).toBe('07:00');
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisTime = '28:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        parisTime,
        'Europe/Paris',
      );
      expect(utcDatetime).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisTime = undefined;
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        parisTime,
        'Europe/Paris',
      );
      expect(utcDatetime).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const fooBarTime = '08:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        fooBarTime,
        'Foo/Bar',
      );
      expect(utcDatetime).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const fooBarTime = '08:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        fooBarTime,
        undefined,
      );
      expect(utcDatetime).toBeUndefined();
    });
  });

  describe('localStringDateTimeToUtcDate', () => {
    it('should convert a summer paris date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate.toISOString()).toBe('2023-06-22T10:00:00.000Z');
    });
    it('should convert a winter paris date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-02-02';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate.toISOString()).toBe('2023-02-02T11:00:00.000Z');
    });
    it('should convert a summer paris date and time to a utc date without dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
        false,
      );
      expect(utcDate.toISOString()).toBe('2023-06-22T11:00:00.000Z');
    });
    it('should convert a tonga date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const tongaDate = '2023-02-02';
      const tongaTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        tongaDate,
        tongaTime,
        'Pacific/Tongatapu',
      );
      expect(utcDate.toISOString()).toBe('2023-02-01T23:00:00.000Z');
    });
    it('should convert a papeete date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const papeeteDate = '2023-02-02';
      const papeeteTime = '15:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        papeeteDate,
        papeeteTime,
        'Pacific/Tahiti',
      );
      expect(utcDate.toISOString()).toBe('2023-02-03T01:00:00.000Z');
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '28:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = undefined;
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Foo/Bar',
      );
      expect(utcDate).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        undefined,
      );
      expect(utcDate).toBeUndefined();
    });
  });

  describe('utcBaseDayFromTime', () => {
    it('should get the utc day of paris at 12:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('12:00', 'Europe/Paris'),
      ).toBe(4);
    });
    it('should get the utc day of paris at 00:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('00:00', 'Europe/Paris'),
      ).toBe(3);
    });
    it('should get the utc day of papeete at 16:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('16:00', 'Pacific/Tahiti'),
      ).toBe(5);
    });
    it('should return undefined if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('28:00', 'Europe/Paris'),
      ).toBeUndefined();
    });
    it('should return undefined if time is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime(undefined, 'Europe/Paris'),
      ).toBeUndefined();
    });
    it('should return undefined if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('12:00', 'Foo/Bar'),
      ).toBeUndefined();
    });
    it('should return undefined if timezone is undefined', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('12:00', undefined),
      ).toBeUndefined();
    });
  });
});
